/*****************************************************************************\
 *                        ANALYSIS PERFORMANCE TOOLS                         *
 *                             ClusteringSuite                               *
 *   Infrastructure and tools to apply clustering analysis to Paraver and    *
 *                              Dimemas traces                               *
 *                                                                           *
 *****************************************************************************
 *     ___     This library is free software; you can redistribute it and/or *
 *    /  __         modify it under the terms of the GNU LGPL as published   *
 *   /  /  _____    by the Free Software Foundation; either version 2.1      *
 *  /  /  /     \   of the License, or (at your option) any later version.   *
 * (  (  ( B S C )                                                           *
 *  \  \  \_____/   This library is distributed in hope that it will be      *
 *   \  \__         useful but WITHOUT ANY WARRANTY; without even the        *
 *    \___          implied warranty of MERCHANTABILITY or FITNESS FOR A     *
 *                  PARTICULAR PURPOSE. See the GNU LGPL for more details.   *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public License  *
 * along with this library; if not, write to the Free Software Foundation,   *
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA          *
 * The GNU LEsser General Public License is contained in the file COPYING.   *
 *                                 ---------                                 *
 *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
\*****************************************************************************/

/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\

  $Id::                                       $:  Id
  $Rev::                                          $:  Revision of last commit
  $Author::                                       $:  Author of last commit
  $Date::                                         $:  Date of last commit

\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#include <BackEnd.h>
#include "TDBSCANWorkerOffline.h"

#if defined(BACKEND_ATTACH)

#include <mpi.h>

int main(int argc, char *argv[])
{
   int rank;

   MPI_Init(&argc, &argv);
   MPI_Comm_rank(MPI_COMM_WORLD, &rank);

   BackEnd *BE = new BackEnd();
   BE->Init(rank);

   BackProtocol *protClustering = new TDBSCANWorkerOffline();
   BE->LoadProtocol( protClustering );

   BE->Loop();

   MPI_Finalize();
   return 0;
}

#else

/**
 * The back-end application loads the TDBSCAN protocol and waits for 
 * the front-end to start the analysis.
 * @param argc Number of arguments.
 * @param argv Array of arguments.
 * @return 0 on success; -1 otherwise.
 */ 
int main(int argc, char *argv[])
{
   BackEnd *BE = new BackEnd();
   BE->Init(argc, argv);

   BackProtocol *protClustering = new TDBSCANWorkerOffline();
   BE->LoadProtocol( protClustering );

   BE->Loop();

   return 0;
}

#endif

